/* $Id: ModeCBC.java,v 1.1 1999/08/22 02:53:20 gelderen Exp $
 *
 * Copyright (C) 1995-1999 Systemics Ltd.
 * on behalf of the Cryptix Development Team. All rights reserved.
 *
 * Use, modification, copying and distribution of this software is subject
 * the terms and conditions of the Cryptix General Licence. You should have
 * received a copy of the Cryptix General License along with this library;
 * if not, you can download a copy from http://www.cryptix.org/ .
 */
package cryptix.jce12.provider.cipher;


import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher; // FIXME: remove? (see below)
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.ShortBufferException;
import javax.crypto.spec.IvParameterSpec;


// FIXME: make explicit
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;


final class ModeCBC extends Mode
{
    /** buffers incomplete blocks */
    private final byte[] buf;


    /** how many bytes our buffer holds */
    private int bufCount;


    /** previous ciphertext block (during decryption only) */
    private final byte[] prevBlock;
    
    
    ModeCBC(BlockCipher cipher) {
        super(cipher);
        buf       = new byte[CIPHER_BLOCK_SIZE];
        prevBlock = new byte[CIPHER_BLOCK_SIZE];
    }


// Implementation
//............................................................................

    final int coreGetOutputSize(int inputLen) {
        return ((bufCount+inputLen)/CIPHER_BLOCK_SIZE)*CIPHER_BLOCK_SIZE;
    }


    final void coreInit(boolean decrypt, Key key, AlgorithmParameterSpec params,
                    SecureRandom random)
    throws InvalidKeyException, InvalidAlgorithmParameterException
    {
        cipher.coreInit(key, decrypt);
        
        if(decrypt) {
            // set IV
            IvParameterSpec iv = (IvParameterSpec)params;
            System.arraycopy(iv.getIV(), 0, prevBlock, 0, CIPHER_BLOCK_SIZE);
            bufCount = 0;
        } else {
            // set IV
            IvParameterSpec iv = (IvParameterSpec)params;
            System.arraycopy(iv.getIV(), 0, buf, 0, CIPHER_BLOCK_SIZE);
            bufCount = 0;
        }
    }


    int coreUpdate(byte[] input, int inputOffset, int inputLen,
               byte[] output, int outputOffset)
    throws ShortBufferException
    {
        if(decrypt) {
            int ret = 0;
            int remainder;
            while(inputLen >= (remainder = CIPHER_BLOCK_SIZE - bufCount)) {
                
                for( int i=0; i<remainder; i++)
                    buf[bufCount++] = input[inputOffset++];

                // encrypt to output
                cipher.coreCrypt(buf, 0, output, outputOffset);
                
                // xor in previous block
                for( int i=0; i<CIPHER_BLOCK_SIZE; i++)
                    output[outputOffset++] ^= prevBlock[i];
                
                // store cipher text as IV for next block
                for( int i=0; i<CIPHER_BLOCK_SIZE; i++)
                    prevBlock[i] = buf[i];
                
                inputLen     -= CIPHER_BLOCK_SIZE;
                ret          += CIPHER_BLOCK_SIZE;
                bufCount      = 0;
            }
            
            for( int i=0; i<inputLen; i++)
                buf[bufCount++] = input[inputOffset++];

            return ret;

        } else {
            int ret = 0;
            int remainder;
            while(inputLen >= (remainder = CIPHER_BLOCK_SIZE - bufCount)) {
                for( int i=0; i<remainder; i++)
                    buf[bufCount++] ^= input[inputOffset++];

                cipher.coreCrypt(buf, 0, buf, 0);
                System.arraycopy(buf, 0, output, outputOffset, CIPHER_BLOCK_SIZE);
                inputLen     -= remainder;
                outputOffset += CIPHER_BLOCK_SIZE;
                ret          += CIPHER_BLOCK_SIZE;
                bufCount      = 0;
            }

            for( int i=0; i<inputLen; i++)
                buf[bufCount++] ^= input[inputOffset++];

            return ret;
        }
        
        
    }
}